@@ -141,7 +141,7 @@ class OutdatedCohortVersion(Exception):
 class TechnicalFaultException(Exception):
     def __str__(self):
         return "Samples not downloaded due to a technical fault. Please check \
-            the arguments passed and try again. Contact Polly Support in case of repeated failure."
+the arguments passed and try again. Contact Polly Support in case of repeated failure."
 
 
 class InvalidDatatypeException(Exception):
@@ -165,6 +165,14 @@ class InvalidParameterException(Exception):
         return f"Empty or Invalid Parameters = {self.parameter}."
 
 
+class IncompatibleDataSource(Exception):
+    def __init__(self, parameter):
+        self.parameter = parameter
+
+    def __str__(self):
+        return f"The datasets cannot be added due to conflicting data sources = {self.parameter}."
+
+
 class InvalidFormatException(Exception):
     def __str__(self):
         return "This file format is not supported."
@@ -260,9 +268,36 @@ class ResourceNotFoundError(BaseExceptionError):
             self.detail = detail
 
 
+class InvalidSchemaJsonException(Exception):
+    def __str__(self):
+        return "Schema error in the json request file; update and try again"
+
+
+class InvalidSyntaxForRequestException(Exception):
+    def __str__(self):
+        return "Invalid syntax for this request was provided"
+
+
+class EmptyPayloadException(Exception):
+    def __str__(self):
+        return "The specified payload could not be found. Inspect and try again."
+
+
+class InvalidJobFunctionParameterException(Exception):
+    def __init__(self, parameter):
+        self.parameter = parameter
+
+    def __str__(self):
+        return (
+            f"The specified {self.parameter} could not be found. Inspect and try again."
+        )
+
+
 def error_handler(response):
     if has_error_message(response):
         title, detail = extract_json_api_error(response)
+        if title == app_err_info.REPOSITORY_LOCKED:
+            detail = app_err_info.REPOSITORY_LOCKED_DETAIL
         raise RequestException(title, detail)
     elif response.status_code == 401:
         raise UnauthorizedException
