@@ -1,3 +1,4 @@
+import base64
 import os
 import re
 import json
@@ -8,6 +9,7 @@ from cloudpathlib import S3Client
 from botocore.exceptions import ClientError
 from cmapPy.pandasGEXpress.parse_gct import parse
 from os import environ
+from jose import jwt
 from polly.errors import (
     BadRequestError,
     error_handler,
@@ -181,6 +183,25 @@ def get_workspace_payload(
     return payload
 
 
+def extract_error_title(error_msg: str) -> str:
+    """Extract error title from error message
+
+    Args:
+        error_msg (str): Whole Error Message in form of string
+
+    Returns:
+        str: Return title of the error message
+    """
+    error_msg = json.loads(error_msg)
+    error = error_msg.get("error")
+    if error is None:
+        error = error_msg.get("errors")[0]
+    if "title" in error:
+        title = error.get("title")
+
+    return title
+
+
 def file_conversion(
     self, repo_info: str, dataset_id: str, format: str, header_mapping: dict
 ) -> None:
@@ -448,6 +469,189 @@ def get_user_details(self):
     return user_details
 
 
+def get_user_details_using_aws_cognito(self):
+    """
+    Summary :
+    gets the user details such as sub, aud post verification
+    using aws cognito
+
+    Returns:
+        dictionary of token keys and values
+    """
+    # getting the userpool details
+    pool_url = f"https://api.{self.session.env}.elucidata.io/userpool"
+    pool_details = self.session.get(pool_url)
+    pool_details_json = pool_details.json()
+    user_pool_region = pool_details_json.get("cognito_user_pool_region", "")
+    user_pool_id = pool_details_json.get("cognito_user_pool", "")
+    # cognito_client_id = pool_details_json.get("cognito_client_id", "")
+    session_cookies_dict = self.session.cookies.get_dict()
+    id_token = session_cookies_dict.get("idToken", "")
+    session_headers_dict = self.session.headers
+    cookie = session_headers_dict.get("Cookie", "")
+    # cookie is a string
+    if "refreshToken=" in cookie:
+        refresh_token = cookie.split("refreshToken=")[1]
+    else:
+        print("refresh token not presen in header cookie")
+        refresh_token = ""
+
+    # Getting Headers
+    # headers = jwt.get_unverified_header(id_token)
+    kid = get_value_from_idToken(id_token, "kid")
+
+    # Getting Pool Url
+    verify_url = generate_aws_pool_url(user_pool_region, user_pool_id)
+
+    kargs = {"issuer": verify_url}
+    kargs["options"] = {"verify_at_hash": False}
+    kargs["audience"] = get_value_from_idToken(id_token, "aud")
+    keys = aws_key_dict(user_pool_region, user_pool_id)
+
+    # Get Public Key
+    public_key = get_public_key(keys, kid)
+
+    decoded_token = jwt.decode(id_token, public_key, **kargs)
+    decoded_token["id_token"] = id_token
+    decoded_token["refresh_token"] = refresh_token
+
+    return decoded_token
+
+
+def generate_aws_pool_url(user_pool_region, user_pool_id):
+    verify_url = "https://cognito-idp.{}.amazonaws.com/{}".format(
+        user_pool_region, user_pool_id
+    )
+    return verify_url
+
+
+def get_public_key(keys, kid):
+    """
+    Getting public key in pem format from Id Token
+
+    Parameters
+    ----------
+    keys: Dict
+        JWT headers keys
+    kid: String
+        public key identifier
+
+    Returns
+    ----------
+    pubk_bytes: String (PEM)
+        Public key in pem format
+    """
+
+    key = keys[kid]
+    return key
+
+
+def aws_key_dict(region, user_pool_id):
+    """
+    Fetches the AWS JWT validation file (if necessary) and then converts
+    this file into a keyed dictionary that can be used to validate a web-token
+    we've been passed
+
+    Parameters
+    ----------
+    aws_user_pool: String
+        AWS Cognito user pool ID
+    aws_region: String
+        AWS Cognito user pool region
+
+    Returns:
+    -------
+    dict:
+        Contains decoded token dict
+    """
+    filename = "/tmp/" + "aws_{}.json".format(user_pool_id)
+
+    if not os.path.isfile(filename):
+        # If we can't find the file already, try to download it.
+        aws_data = requests.get(
+            ("https://cognito-idp.{}.amazonaws.com/{}".format(region, user_pool_id))
+            + "/.well-known/jwks.json"
+        )
+        aws_jwt = json.loads(aws_data.text)
+        with open(filename, "w+") as json_data:
+            json_data.write(aws_data.text)
+            json_data.close()
+
+    else:
+        with open(filename) as json_data:
+            aws_jwt = json.load(json_data)
+            json_data.close()
+
+    # We want a dictionary keyed by the kid, not a list.
+    result = {}
+    for item in aws_jwt["keys"]:
+        result[item["kid"]] = item
+
+    return result
+
+
+def get_value_from_idToken(id_token: str, key: str) -> dict:
+    decoded_id_token_dict = decode_id_token(id_token)
+    return decoded_id_token_dict.get(key, "")
+
+
+def decode_id_token(tokenID: str) -> dict:
+    """
+    Summary: decodes the token id
+    this method is to decode the tokenID which consists of 3 parts.
+    the first 2 parts can be base64 decoded
+    first part contains the kid and alg details.
+    second part contains cognito:user details (AWS) such as sub, username
+    aud, name, exp, email etc.
+
+    Args:
+        tokenId : str
+
+    Returns:
+        dict of keys and values from decoded idToken
+    """
+    decoded_idToken_dict = {}
+    try:
+        for element in tokenID.split(".")[:2]:
+            # the element if lacking "padding" can give 'incorrect padding' error.
+            # to handle that we have add the extra padding of '='
+            # the len of the element should always be a multiple of 4.
+            # https://stackoverflow.com/questions/2941995/python-ignore-incorrect-padding-error-when-base64-decoding
+
+            decoded_str = base64.b64decode(element + "=" * (-len(element) % 4))
+            decoded_idToken_dict.update(json.loads(decoded_str.decode("utf-8")))
+    except Exception as err:
+        print("something wrong with idToken")
+        raise err
+    return decoded_idToken_dict
+
+
+def makeRequestCookieForPollyJob(job_data: json) -> json:
+
+    """
+    Generates the cookie required request for polly job API
+
+    Args:
+        Job data : Dict
+    Returns:
+        request_cookie: generated cookie string
+    """
+
+    secret_env_vars = job_data.get("secret_env", "")
+
+    aud = secret_env_vars.get("POLLY_AUD", "")
+    sub = secret_env_vars.get("POLLY_SUB", "")
+    idToken = secret_env_vars.get("POLLY_ID_TOKEN", "")
+    refreshToken = secret_env_vars.get("POLLY_REFRESH_TOKEN", "")
+    idTokenCookie = f"CognitoIdentityServiceProvider.{aud}.{sub}.idToken={idToken}"
+    refreshCookie = (
+        f"CognitoIdentityServiceProvider.{aud}.{sub}.refreshToken={refreshToken}"
+    )
+    request_coookie = f"{idTokenCookie};{refreshCookie}"
+    # print("returning request_coookie: " + request_coookie)
+    return request_coookie
+
+
 def workspaces_permission_check(self, workspace_id) -> bool:
     """
     Function to check access of a user for a given workspace id.
@@ -509,3 +713,37 @@ def return_entity_type(data_source: str, cohort_info: json) -> str:
             elif dict["file_structure"] == "multiple":
                 entity_type = "sample"
     return entity_type
+
+
+def get_files_in_dir(path_to_dir: str) -> list:
+    """
+    returns the files in a given directory
+
+    Arguments:
+        path_to_dir: str
+
+    Returns:
+        list of files in dir : list
+    """
+    directory = os.fsencode(path_to_dir)
+    file_names = os.listdir(directory)
+    return file_names
+
+
+def parseInt(sin):
+    """
+    parsed the value passed as int  as done by js.
+    python equivalent of js parseInt
+    example:
+        parseInt("100n")  = 100
+        parseInt("400m")  = 400
+
+    Arguments:
+        sin -- value to be parsed as int
+
+    Returns:
+        int
+    """
+    m = re.search(r"^(\d+)[.,]?\d*?", str(sin))
+    parsed_int = int(m.groups()[-1]) if m and not callable(sin) else None
+    return parsed_int
